@@ -1,4 +1,4 @@
-/* Copyright (C) 2002 Timo Sirainen */
+/* Copyright (c) 2002-2008 Dovecot authors, see the included COPYING file */
 
 #include "common.h"
 #include "ioloop.h"
@@ -19,22 +19,19 @@ struct cmd_idle_context {
 	struct client_command_context *cmd;
 
 	struct imap_sync_context *sync_ctx;
-	struct timeout *idle_to, *keepalive_to;
-	uint32_t dummy_seq;
+	struct timeout *keepalive_to;
 
 	unsigned int manual_cork:1;
-	unsigned int idle_timeout:1;
 	unsigned int sync_pending:1;
 };
 
 static bool cmd_idle_continue(struct client_command_context *cmd);
 
-static void idle_finish(struct cmd_idle_context *ctx, bool done_ok)
+static void
+idle_finish(struct cmd_idle_context *ctx, bool done_ok, bool free_cmd)
 {
 	struct client *client = ctx->client;
 
-	if (ctx->idle_to != NULL)
-		timeout_remove(&ctx->idle_to);
 	if (ctx->keepalive_to != NULL)
 		timeout_remove(&ctx->keepalive_to);
 
@@ -44,41 +41,29 @@ static void idle_finish(struct cmd_idle_context *ctx, bool done_ok)
 	}
 
 	o_stream_cork(client->output);
-
-	if (ctx->dummy_seq != 0) {
-		/* outlook idle workaround */
-		client_send_line(client,
-			t_strdup_printf("* %u EXPUNGE", ctx->dummy_seq));
-	}
-
-	io_remove(&client->io);
+	if (client->io != NULL)
+		io_remove(&client->io);
 
 	if (client->mailbox != NULL)
-		mailbox_notify_changes(client->mailbox, 0, NULL, NULL);
+		mailbox_notify_changes_stop(client->mailbox);
 
-	client->idling = FALSE;
 	if (done_ok)
 		client_send_tagline(ctx->cmd, "OK Idle completed.");
 	else
 		client_send_tagline(ctx->cmd, "BAD Expected DONE.");
 
 	o_stream_uncork(client->output);
-
-	client->bad_counter = 0;
-	_client_reset_command(client);
-
-	/* we may have read something else besides "DONE" into the buffer */
-	if (client->input_pending && !client->handling_input)
-		_client_input(client);
+	if (free_cmd)
+		client_command_free(&ctx->cmd);
 }
 
-static void idle_client_input(void *context)
+static void idle_client_input(struct cmd_idle_context *ctx)
 {
-        struct cmd_idle_context *ctx = context;
 	struct client *client = ctx->client;
 	char *line;
 
 	client->last_input = ioloop_time;
+	timeout_reset(client->to_idle);
 
 	switch (i_stream_read(client->input)) {
 	case -1:
@@ -87,7 +72,8 @@ static void idle_client_input(void *context)
 		return;
 	case -2:
 		client->input_skip_line = TRUE;
-		idle_finish(ctx, FALSE);
+		idle_finish(ctx, FALSE, TRUE);
+		client_continue_pending_input(&client);
 		return;
 	}
 
@@ -102,51 +88,27 @@ static void idle_client_input(void *context)
 		if (client->input_skip_line)
 			client->input_skip_line = FALSE;
 		else {
-			idle_finish(ctx, strcmp(line, "DONE") == 0);
+			idle_finish(ctx, strcmp(line, "DONE") == 0, TRUE);
+			client_continue_pending_input(&client);
 			break;
 		}
 	}
 }
 
-static void idle_send_fake_exists(struct cmd_idle_context *ctx)
+static void keepalive_timeout(struct cmd_idle_context *ctx)
 {
-	struct client *client = ctx->client;
-
-	ctx->dummy_seq = client->messages_count+1;
-	client_send_line(client,
-			 t_strdup_printf("* %u EXISTS", ctx->dummy_seq));
-	mailbox_notify_changes(client->mailbox, 0, NULL, NULL);
-}
-
-static void idle_timeout(void *context)
-{
-	struct cmd_idle_context *ctx = context;
-
-	/* outlook workaround - it hasn't sent anything for a long time and
-	   we're about to disconnect unless it does something. send a fake
-	   EXISTS to see if it responds. it's expunged later. */
-
-	timeout_remove(&ctx->idle_to);
-
-	if (ctx->sync_ctx != NULL) {
-		/* we're already syncing.. do this after it's finished */
-		ctx->idle_timeout = TRUE;
-		return;
-	}
-
-	idle_send_fake_exists(ctx);
-}
-
-static void keepalive_timeout(void *context)
-{
-	struct cmd_idle_context *ctx = context;
-
-	if (ctx->client->output_pending) {
+	if (ctx->client->output_lock != NULL) {
 		/* it's busy sending output */
 		return;
 	}
 
+	/* Sending this keeps NATs/stateful firewalls alive. Sending this
+	   also catches dead connections. */
 	client_send_line(ctx->client, "* OK Still here");
+	/* Make sure idling connections don't get disconnected. There are
+	   several clients that really want to IDLE forever and there's not
+	   much harm in letting them do so. */
+	timeout_reset(ctx->client->to_idle);
 }
 
 static void idle_sync_now(struct mailbox *box, struct cmd_idle_context *ctx)
@@ -158,10 +120,8 @@ static void idle_sync_now(struct mailbox *box, struct cmd_idle_context *ctx)
 	cmd_idle_continue(ctx->cmd);
 }
 
-static void idle_callback(struct mailbox *box, void *context)
+static void idle_callback(struct mailbox *box, struct cmd_idle_context *ctx)
 {
-        struct cmd_idle_context *ctx = context;
-
 	if (ctx->sync_ctx != NULL)
 		ctx->sync_pending = TRUE;
 	else {
@@ -175,6 +135,11 @@ static bool cmd_idle_continue(struct client_command_context *cmd)
 	struct client *client = cmd->client;
 	struct cmd_idle_context *ctx = cmd->context;
 
+	if (cmd->cancel) {
+		idle_finish(ctx, FALSE, FALSE);
+		return TRUE;
+	}
+
 	if (ctx->manual_cork)  {
 		/* we're coming from idle_callback instead of a normal
 		   I/O handler, so we'll have to do corking manually */
@@ -188,21 +153,19 @@ static bool cmd_idle_continue(struct client_command_context *cmd)
 				ctx->manual_cork = FALSE;
 				o_stream_uncork(client->output);
 			}
+			cmd->state = CLIENT_COMMAND_STATE_WAIT_OUTPUT;
 			return FALSE;
 		}
 
 		if (imap_sync_deinit(ctx->sync_ctx) < 0) {
 			client_send_untagged_storage_error(client,
 				mailbox_get_storage(client->mailbox));
-			mailbox_notify_changes(client->mailbox, 0, NULL, NULL);
+			mailbox_notify_changes_stop(client->mailbox);
 		}
 		ctx->sync_ctx = NULL;
 	}
 
-	if (ctx->idle_timeout) {
-		/* outlook workaround */
-		idle_send_fake_exists(ctx);
-	} else if (ctx->sync_pending) {
+	if (ctx->sync_pending) {
 		/* more changes occurred while we were sending changes to
 		   client */
 		idle_sync_now(client->mailbox, ctx);
@@ -210,7 +173,7 @@ static bool cmd_idle_continue(struct client_command_context *cmd)
 		   so we return here instead of doing everything twice. */
 		return FALSE;
 	}
-        client->output_pending = FALSE;
+	cmd->state = CLIENT_COMMAND_STATE_WAIT_INPUT;
 
 	if (ctx->manual_cork) {
 		ctx->manual_cork = FALSE;
@@ -218,7 +181,7 @@ static bool cmd_idle_continue(struct client_command_context *cmd)
 	}
 
 	if (client->output->closed) {
-		idle_finish(ctx, FALSE);
+		idle_finish(ctx, FALSE, FALSE);
 		return TRUE;
 	}
 	if (client->io == NULL) {
@@ -241,11 +204,6 @@ bool cmd_idle(struct client_command_context *cmd)
 	ctx->cmd = cmd;
 	ctx->client = client;
 
-	if ((client_workarounds & WORKAROUND_OUTLOOK_IDLE) != 0 &&
-	    client->mailbox != NULL) {
-		ctx->idle_to = timeout_add((CLIENT_IDLE_TIMEOUT - 60) * 1000,
-					   idle_timeout, ctx);
-	}
 	ctx->keepalive_to = timeout_add(KEEPALIVE_TIMEOUT * 1000,
 					keepalive_timeout, ctx);
 
@@ -264,8 +222,6 @@ bool cmd_idle(struct client_command_context *cmd)
 	client->io = io_add(i_stream_get_fd(client->input),
 			    IO_READ, idle_client_input, ctx);
 
-	client->idling = TRUE;
-	client->command_pending = TRUE;
 	cmd->func = cmd_idle_continue;
 	cmd->context = ctx;
 
