@@ -1,4 +1,4 @@
-/* Copyright (C) 2002 Timo Sirainen */
+/* Copyright (c) 2002-2008 Dovecot authors, see the included COPYING file */
 
 /*
  * Merge sort code in sort_nodes() is copyright 2001 Simon Tatham.
@@ -93,12 +93,12 @@ static void mail_thread_finish(struct thread_context *ctx);
 static void mail_thread_deinit(struct thread_context *ctx)
 {
 	if (ctx->msgid_hash != NULL)
-		hash_destroy(ctx->msgid_hash);
+		hash_destroy(&ctx->msgid_hash);
 	if (ctx->subject_hash != NULL)
-		hash_destroy(ctx->subject_hash);
+		hash_destroy(&ctx->subject_hash);
 
-	pool_unref(ctx->temp_pool);
-	pool_unref(ctx->pool);
+	pool_unref(&ctx->temp_pool);
+	pool_unref(&ctx->pool);
 }
 
 int imap_thread(struct client_command_context *cmd, const char *charset,
@@ -139,8 +139,11 @@ int imap_thread(struct client_command_context *cmd, const char *charset,
 	headers_ctx = mailbox_header_lookup_init(client->mailbox,
 						 wanted_headers);
 	mail = mail_alloc(ctx->t, MAIL_FETCH_DATE, headers_ctx);
-	while (mailbox_search_next(ctx->search_ctx, mail) > 0)
-		mail_thread_input(ctx, mail);
+	while (mailbox_search_next(ctx->search_ctx, mail) > 0) {
+		T_BEGIN {
+			mail_thread_input(ctx, mail);
+		} T_END;
+	}
 
 	mail_free(&mail);
 
@@ -149,7 +152,7 @@ int imap_thread(struct client_command_context *cmd, const char *charset,
 	o_stream_send_str(client->output, "\r\n");
 
 	ret = mailbox_search_deinit(&ctx->search_ctx);
-	if (mailbox_transaction_commit(&ctx->t, 0) < 0)
+	if (mailbox_transaction_commit(&ctx->t) < 0)
 		ret = -1;
 
 	mailbox_header_lookup_deinit(&headers_ctx);
@@ -444,26 +447,28 @@ static void mail_thread_input(struct thread_context *ctx, struct mail *mail)
 	struct node *node;
 	time_t sent_date;
 
-	t_push();
-
-	sent_date = mail_get_date(mail, NULL);
-	if (sent_date == (time_t)-1)
+	if (mail_get_date(mail, &sent_date, NULL) < 0)
 		sent_date = 0;
 	if (sent_date == 0) {
-		sent_date = mail_get_received_date(mail);
-		if (sent_date == (time_t)-1)
+		if (mail_get_received_date(mail, &sent_date) < 0)
 			sent_date = 0;
 	}
 
-	message_id = mail_get_first_header(mail, "message-id");
+	if (mail_get_first_header(mail, "message-id", &message_id) < 0)
+		message_id = NULL;
 	node = update_message(ctx, get_msgid(&message_id), sent_date,
 			      ctx->id_is_uid ? mail->uid : mail->seq);
 
 	/* link references */
-	references = mail_get_first_header(mail, "references");
+	if (mail_get_first_header(mail, "references", &references) < 0)
+		references = NULL;
+
 	if (!link_references(ctx, node, references)) {
-		in_reply_to = mail_get_first_header(mail, "in-reply-to");
-		refid = in_reply_to == NULL ? NULL : get_msgid(&in_reply_to);
+		if (mail_get_first_header(mail, "in-reply-to",
+					  &in_reply_to) <= 0)
+			refid = NULL;
+		else
+			refid = get_msgid(&in_reply_to);
 
 		if (refid != NULL)
 			link_node(ctx, refid, node, TRUE);
@@ -473,8 +478,6 @@ static void mail_thread_input(struct thread_context *ctx, struct mail *mail)
 				unlink_child(ctx, node, TRUE);
 		}
 	}
-
-	t_pop();
 }
 
 static struct node *find_last_child(struct node *node)
@@ -569,7 +572,7 @@ static struct node *sort_nodes(struct node *list)
 		tail = NULL;
 
 		nmerges = 0;  /* count number of merges we do in this pass */
-		while (p != 0) {
+		while (p != NULL) {
 			nmerges++;  /* there exists a merge to be done */
 
 			/* step `insize' places along from p */
@@ -693,16 +696,17 @@ static void gather_base_subjects(struct thread_context *ctx)
 
 		if (!ctx->id_is_uid)
 			seq = id;
-		else {
-			if (mailbox_get_uids(ctx->box, id, id, &seq, &seq) < 0)
-				seq = 0;
-		}
+		else
+			mailbox_get_uids(ctx->box, id, id, &seq, &seq);
 
-		if (seq != 0 && mail_set_seq(ctx->mail, seq) == 0) {
-			t_push();
-                        subject = mail_get_first_header(ctx->mail, "subject");
-			add_base_subject(ctx, subject, node);
-			t_pop();
+		if (seq != 0) {
+			mail_set_seq(ctx->mail, seq);
+			if (mail_get_first_header(ctx->mail, "subject",
+						  &subject) > 0) {
+				T_BEGIN {
+					add_base_subject(ctx, subject, node);
+				} T_END;
+			}
 		}
 	}
 
@@ -935,18 +939,15 @@ static void mail_thread_finish(struct thread_context *ctx)
 	/* (2) save root nodes and drop the msgids */
 	iter = hash_iterate_init(ctx->msgid_hash);
 	while (hash_iterate(iter, &key, &value)) {
-		struct node *node = value;
-
+		node = value;
 		if (node->parent == NULL)
 			add_root(ctx, node);
 	}
-	hash_iterate_deinit(iter);
+	hash_iterate_deinit(&iter);
 
 	/* drop the memory allocated for message-IDs and msgid_hash,
 	   reuse their memory for base subjects */
-	hash_destroy(ctx->msgid_hash);
-	ctx->msgid_hash = NULL;
-
+	hash_destroy(&ctx->msgid_hash);
 	p_clear(ctx->temp_pool);
 
 	if (ctx->root_node.first_child == NULL) {
@@ -973,7 +974,7 @@ static void mail_thread_finish(struct thread_context *ctx)
 	merge_subject_threads(ctx);
 
 	/* (6) Sort and send replies */
-	t_push();
-	send_roots(ctx);
-	t_pop();
+	T_BEGIN {
+		send_roots(ctx);
+	} T_END;
 }
