@@ -173,6 +173,7 @@ get_var_expand_table(const char *protocol,
 		     const char *local_ip, const char *remote_ip,
 		     pid_t pid, uid_t uid)
 {
+#define VAR_EXPAND_HOME_IDX 4
 	static struct var_expand_table static_tab[] = {
 		{ 'u', NULL },
 		{ 'n', NULL },
@@ -195,8 +196,7 @@ get_var_expand_table(const char *protocol,
 	tab[2].value = user == NULL ? NULL : strchr(user, '@');
 	if (tab[2].value != NULL) tab[2].value++;
 	tab[3].value = t_str_ucase(protocol);
-	tab[4].value = home != NULL ? home :
-		"/HOME_DIRECTORY_USED_BUT_NOT_GIVEN_BY_USERDB";
+	tab[VAR_EXPAND_HOME_IDX].value = home;
 	tab[5].value = local_ip;
 	tab[6].value = remote_ip;
 	tab[7].value = dec2str(pid);
@@ -205,6 +205,15 @@ get_var_expand_table(const char *protocol,
 	return tab;
 }
 
+static bool
+has_missing_used_home(const char *str, const struct var_expand_table *table)
+{
+	i_assert(table[VAR_EXPAND_HOME_IDX].key == 'h');
+
+	return table[VAR_EXPAND_HOME_IDX].value == NULL &&
+		var_has_key(str, 'h');
+}
+
 static const char *
 expand_mail_env(const char *env, const struct var_expand_table *table)
 {
@@ -224,6 +233,11 @@ expand_mail_env(const char *env, const struct var_expand_table *table)
 		str_append_c(str, *env++);
 	}
 
+	if (has_missing_used_home(env, table)) {
+		i_fatal("userdb didn't return a home directory, "
+			"but mail location used it (%%h): %s", env);
+	}
+
 	/* expand %vars */
 	var_expand(str, env, table);
 	return str_c(str);
@@ -401,6 +415,12 @@ mail_process_set_environment(struct settings *set, const char *mail,
 		str_truncate(str, 0);
 		var_expand(str, envs[i+1], var_expand_table);
 
+		if (has_missing_used_home(envs[i+1], var_expand_table)) {
+			i_error("userdb didn't return a home directory, "
+				"but it's used in plugin setting %s: %s",
+				envs[i], envs[i+1]);
+		}
+
 		env_put(t_strconcat(t_str_ucase(envs[i]), "=",
 				    str_c(str), NULL));
 	}
@@ -831,6 +851,7 @@ create_mail_process(enum process_type process_type, struct settings *set,
 
 	addr = net_ip2addr(remote_ip);
 	env_put(t_strconcat("IP=", addr, NULL));
+	env_put(t_strconcat("LOCAL_IP=", net_ip2addr(local_ip), NULL));
 
 	if (!set->verbose_proctitle)
 		title[0] = '\0';
