@@ -107,6 +107,12 @@ static bool client_handle_args(struct pop3_client *client,
 			destuser = *args + 9;
 		else if (strncmp(*args, "pass=", 5) == 0)
 			pass = *args + 5;
+		else if (strncmp(*args, "user=", 5) == 0) {
+			/* already handled in login-common */
+		} else if (auth_debug) {
+			i_info("Ignoring unknown passdb extra field: %s",
+			       *args);
+		}
 	}
 
 	if (destuser == NULL)
@@ -146,6 +152,7 @@ static bool client_handle_args(struct pop3_client *client,
 			io_remove(&client->io);
 		client->io = io_add(client->common.fd, IO_READ,
 				    client_input, client);
+		client_input(client);
 	}
 	return TRUE;
 }
@@ -189,6 +196,7 @@ static void sasl_callback(struct client *_client, enum sasl_server_reply reply,
 				io_remove(&client->io);
 			client->io = io_add(client->common.fd, IO_READ,
 					    client_input, client);
+			client_input(client);
 		}
 		break;
 	case SASL_SERVER_REPLY_MASTER_FAILED:
